package com.aliyun.odps.spark.examples

import org.apache.spark.sql.SparkSession
import com.aliyun.odps.TableSchema
import com.aliyun.odps.data.Record
import com.aliyun.odps.spark.examples.myUtils.ParamUtils
import com.aliyun.odps.spark.examples.myUtils.env
import com.google.common.collect.ListMultimap
import com.tzld.piaoquan.recommend.feature.domain.video.base.RequestContext
import com.tzld.piaoquan.recommend.feature.domain.video.feature.VlogShareLRFeatureExtractor
import examples.dataloader.RequestContextOffline

import scala.collection.JavaConversions._

object makedata_01_readhdfs {
  def main(args: Array[String]) {
    val spark = SparkSession
      .builder()
      .appName(this.getClass.getName)
      .getOrCreate()
    val sc = spark.sparkContext

    // 1 读取参数
    val param = ParamUtils.parseArgs(args)
    val partition = param.getOrElse("partition", "dt=20231220")
    val tablePart = param.getOrElse("tablePart", "16").toInt


    // 2 读取训练数据
    val odpsOps = env.getODPS(sc)
    val project = "loghubods"
    val table = "alg_recsys_view_sample"
    val odpsData = odpsOps.readTable(project = project,
      table = table,
      partition = partition,
      transfer = read,
      numPartition = tablePart)
    val hdfsPath = "/dw/recommend/model/share_ratio_samples/" + partition
    odpsData.saveAsTextFile(hdfsPath)
  }

  def read(record: Record, schema: TableSchema): String = {
    val labelName = "share_ornot"
    val label = record.getString(labelName)
    val newLabel = if (label == null || label == "1")
      "0"
    else
      "1"
    val result = singleParse(record)
    result
  }

  def singleParse(record: Record): String = {
    //1 处理标签
    val label: String = record.getString("share_ornot")
    val newLabel = if ("1".equals(label)) "0" else "1"
    //2 处理特征
    val reqContext: RequestContextOffline = new RequestContextOffline()
    reqContext.putUserFeature(record)
    reqContext.putItemFeature(record)
    reqContext.putSceneFeature(record)



    val bytesFeatureExtractor: VlogShareLRFeatureExtractor = new VlogShareLRFeatureExtractor()
    bytesFeatureExtractor.getUserFeatures(userBytesFeature)
    bytesFeatureExtractor.getItemFeature(videoBytesFeature)
    bytesFeatureExtractor.getContextFeatures(requestContextBytesFeature)
    val featureMap = bytesFeatureExtractor.getFeatures
    newLabel + "\t" + featureMap.entries().map(r => r.getValue.getIdentifier + ":1").mkString("\t")
  }


}